package pinball;

import pinball.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.adjustable.*;
import pinball.buffer.*;
import pinball.part.*;
import pinball.menuevent.*;

import pinball.player.standard.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;

/** 
 * This sets up the window that displays the image. 
 * It also has buttons to control the start and stop of the program.
 */
public class MainMenu extends Frame{
	/** Width of the image. */	
	protected	int width;
	
	/** Height of the image. */	
	protected int height;
	
	/** The camera watching the game. */
	protected	Camera Qcam;

	/** The canvas for the image in the window. */
	protected	DrawingCanvas picture;	
	
	/** The post strategy thread. */
	protected Post post;

	/** The frame capturing thread. */
	protected	FrameGrabber grabber;
	
	/** The frame analyzing thread. */
	protected Analyze analysis;
	
	/** The frame strategy thread. */
	protected Strategy strategy;
	

	/* Buttons in the window. */
		
	/** Exit the system. */
	protected Button exit;
	
	/** Begin the system. */
	protected Button start;
	
	/** Display the changes in the image. */	
	protected Button verbose;
	
	/** Ball is out of play. */
	protected Button ball;
	
	/** Launch the ball. */	
	protected Button launch;
	
	/** Insert a coin. */
	protected Button coin;
	
	/** Start the pinball game. */
	protected Button startGame;
	
	/** Refind the flippers. */
	protected Button refindFlippers;
	
	/** Pause the system. */
	protected Button pause;	

	/** Excess size for the window. */
	protected	static final int boarder=500;
	
	/** The first of the list of information buffers. */
	protected Buffer buffer;
	
	/** The area on the window to display infomation about the game. */
	protected TextArea gameStats;
	
	/* The panels in the window. */

	/** Top is the image. */
	protected Panel 	top=new Panel();
	
	/** middle is the statistics. */
	protected Panel	middle=new Panel();
	
	/** bottom has the buttons. */
	protected Panel	bottom=new Panel();
	
	/** The buttons for controlling the system. */
	protected Panel	systemButtons=new Panel();
	
	/** The buttons for controlling the game. */			
	protected Panel	gameButtons=new Panel();

	/** For extended classes. */
	public MainMenu(){}

	/**
	 * Create the window and get the camera's settings
	 *
	 * @param Qcam The camera.
 	 * @param player The player of the game.
	 * @param stratThread Defines how the strategy is to player. Either local, client or server.
	 */
	public MainMenu(Camera Qcam, StdPlayer player, String stratThread) {
	        System.err.println("Starting");	   
	   
		this.Qcam=Qcam;
	   
		/* Get the image size. */
		width=Qcam.maxWidth*Qcam.zoomSize();
		height=Qcam.maxHeight*Qcam.zoomSize();
	
		System.err.println("Begining");  
	
		gameStats=new TextArea(5,30);

	    	/* Make the image buffers. */
		buffer=player.makeBuffers(GameOptions.bufferSize(),Qcam);
		
	    	/* Create the analyzing thread. */
		analysis=player.analyze(buffer);
	
	
		/* Create the Local strategy thread. */
		if(stratThread.equals("l"))
			strategy=player.localStrategy(buffer,gameStats,Qcam);
		/* Create the Server strategy thread. */			
		else if(stratThread.equals("s"))
			strategy=player.serverStrategy(buffer,gameStats,Qcam);
		/* Wrong strategy thread. */
		else {
			System.err.println("Error: Wrong Strategy Thread Command");
			System.exit(1);
		}

		/* Create the analyzing thread. */
		grabber=player.frameGrabber(Qcam,buffer);
		
		/* Create the post thread. */
		if (GameOptions.post()) {	
			post=player.post(buffer);
			if(GameOptions.display()) 
				picture=post.canvas(Qcam.xSize(),Qcam.ySize());
			
		}
		

		/* Make the window. */
	    	this.createWindow(width+boarder,height+boarder+50);

	}
	
	/** 
	 * Create the image window with canvas for image. 
	 *
	 * @param width The width of the window.
	 * @param height The height of the window.
	 */
	private void createWindow (int width, int height) {
		
	
		System.err.println("Going to create a window");
		this.setLayout(new GridLayout(3,1));
		
		/* The panels of the window. */
		this.add(top);
		this.add(middle);
		this.add(bottom);	
		
		top.setLayout(new FlowLayout(FlowLayout.CENTER));
		middle.setLayout(new FlowLayout(FlowLayout.CENTER));
		bottom.setLayout(new GridLayout(2,1));
		
		bottom.add(systemButtons);
		bottom.add(gameButtons);				
		
		setSize(width,height);
		setTitle("QuickCam Image");
														
		/* Add the image canvas to the window. */
		if (GameOptions.display()) {
			top.add("Center",picture);
		}
		else top.add(new Label(""));
		

		/* Make the buttons. */
		exit=new Button();
		start=new Button();
		verbose=new Button();
		ball=new Button();
		launch=new Button();
		coin=new Button();
		startGame=new Button();
		refindFlippers=new Button();
		pause=new Button();
		
		exit.setLabel("Exit");
		start.setLabel("Start");
		verbose.setLabel("Difference");
		
		ball.setLabel("Out of Play");
		launch.setLabel("Launch Ball");
		startGame.setLabel("Start Game");
		coin.setLabel("Insert Coin");
		refindFlippers.setLabel("Refind Flippers");
		pause.setLabel("Pause System");
		
		exit.setBackground(Color.lightGray);
		start.setBackground(Color.lightGray);
		verbose.setBackground(Color.lightGray);
		ball.setBackground(Color.lightGray);
		launch.setBackground(Color.lightGray);
		coin.setBackground(Color.lightGray);
		startGame.setBackground(Color.lightGray);
		refindFlippers.setBackground(Color.lightGray);
		pause.setBackground(Color.lightGray);

		middle.add(gameStats);

		
		/* Place the buttons in the window. */
				
		systemButtons.add(start);		
		systemButtons.add(exit);
		systemButtons.add(verbose);
		systemButtons.add(pause);
		
		gameButtons.add(startGame);
		gameButtons.add(launch);
		gameButtons.add(coin);
		gameButtons.add(ball);
		gameButtons.add(refindFlippers);
		
				
		System.err.println("Window made.");

	}
			
	/**
	 * Create the threads, add the button events and display the window.
	 */		       		       	
	public void begin() {
	
		PinballStart pinballStart;
		PinballBallLaunch launchBall;
	
		System.err.println("Adding events.");
	
		/* Button events. */
		exit.addActionListener(new PinballExit(Qcam));
		start.addActionListener(pinballStart=new PinballStart(this, post, grabber, analysis, strategy, start));
		verbose.addActionListener(new PinballVerbose(verbose));
		ball.addActionListener(new PinballBallOut(ball));
		launch.addActionListener(launchBall=new PinballBallLaunch(launch));
		coin.addActionListener(new PinballCoin(coin));
		startGame.addActionListener(new PinballGameStart(startGame));

		refindFlippers.addActionListener(new PinballRefindFlippers(refindFlippers));

		pause.addActionListener(new PinballPause(pause));

		System.err.println("Events added.");
		
		/* If not in batch mode, then place the window on the screen. */	
		if(!GameOptions.batchMode()) {
			
			setVisible(true);
			toFront();
			System.err.println("Window Done.");
		}
		
		if(GameOptions.batchMode()) {
			pinballStart.start();
			launchBall.launch();
		}
	}
}


	


						





