package pinball;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.io.*;

/** 
 * The time stamp for the various stages of the game. 
 */
public class TimeStamp implements Serializable{
	/** Used to keep the same serialized version. */
	static final long serialVersionUID = 4469525270028776046L;

	/** Time at start of program. */
	private long startTime; 
	
	/** Time at end of program. */
	private long endTime;

	/** Time started grabing frames. */
	private long startgrabTime;

	/** Time started analyzing frames. */
	private long startanalyzeTime;
	
	/** Time started displaying frames. */	
	private long startdisplayTime;

	/** End time for pass through of grabing of frames. */
	private long endgrabTime;
	
	/** End time for pass through of analyzing of frames. */	
	private long endanalyzeTime;
	
	/** End time for pass through of displaying of frames. */	
	private long enddisplayTime;

	/** Elapsed time spent grabing. */
	private static long elapsedgrabTime;	
	
	/** Elapsed time spent analyzing. */
	private static long elapsedanalyzeTime;
	
	/** Elapsed time spent grabing. */	
	private static long elapseddisplayTime;

	/** Average time spent in grab thread. */
	private long avggrabTime;

	/** Average time spent in analyze thread. */
	private long avganalyzeTime;

	/** Average time spent in display thread. */
	private long avgdisplayTime;
	
	/** Total number of times through grab thread. */
	private static int totalgrabs=0;
	
	/** Total number of times through analyze thread. */
	private static int totalanalyzes=0;
	
	/** Total number of times through display thread. */	
	private static int totaldisplays=0;
	
	/** Time spent in the grab thread. */
	private long grabTime=0;
	
	/** Time spent in the analyze thread. */
	private long analyzeTime=0;
	
	/** Time spent in the display thread. */
	private long displayTime=0;
	
	/** Begining of time through the threads. */
	private long lastTime;
	
	/** Frame rate of images into system. */
	private float frameRate=0;
	
	/** Time of first grab. */
	private static long firstGrabTime;
	
	/** Time elapsed in the system. */
	private float elapsed;
	
	/** The frame rate for this frame. */
	private float ThisFPS=0;
	
	/** 
	 * Create a new timeStamp. 
	 */
	public TimeStamp () {
		startTime=System.currentTimeMillis();
		endTime=startTime;
		
		startgrabTime=startTime;
		startanalyzeTime=startTime;
		startdisplayTime=startTime;

		endgrabTime=startTime;
		endanalyzeTime=startTime;
		enddisplayTime=startTime;

		elapsedgrabTime=0;
		elapsedanalyzeTime=0;
		elapseddisplayTime=0;

		avggrabTime=0;
		avganalyzeTime=0;
		avgdisplayTime=0;
		
		lastTime=startTime;

	}
	
	/** 
	 * Reset the time stamp. Used when checking flippers. Since the flippers pause,
	 *  they effect the time results. So reset the time after each flipper test.
	 */
	public void reset() {
		
		startTime=System.currentTimeMillis();
		firstGrabTime=System.currentTimeMillis();
		endTime=startTime;
		
		startgrabTime=startTime;
		startanalyzeTime=startTime;
		startdisplayTime=startTime;

		endgrabTime=startTime;
		endanalyzeTime=startTime;
		enddisplayTime=startTime;

		elapsedgrabTime=0;
		elapsedanalyzeTime=0;
		elapseddisplayTime=0;

		avggrabTime=0;
		avganalyzeTime=0;
		avgdisplayTime=0;
		
		lastTime=startTime;
	}
	
	/** 
	 * Set the start time for the program.
	 */
	public void startTime() {
		startTime=System.currentTimeMillis();
	}
	
	/** 
	 * Set the time at the end of the program. 
	 */
	public void setEndTime (){
		endTime=System.currentTimeMillis();
	}
	
	/** 
	 * Set the time a frame is grabed. 
	 */
	public void setFirstGrab() {
		firstGrabTime=System.currentTimeMillis();
		totalgrabs=0;
	}
	
	/** 
	 * Start grab. 
	 */
	public void setStartGrabTime (){
		startgrabTime=System.currentTimeMillis();
		ThisFPS=(startgrabTime-lastTime)/GameOptions.bufferSize();
		lastTime=startgrabTime;		
	}

	/** 
	 * Ended a grab. 
	 */
	public void setEndGrabTime (){
		endgrabTime=System.currentTimeMillis();
		elapsedgrabTime+=endgrabTime-startgrabTime;
		grabTime=endgrabTime-startgrabTime;
		totalgrabs++;
	}
	
	/** 
	 * Started to analyzed. 
	 */
	public void setStartAnalyzeTime (){
		startanalyzeTime=System.currentTimeMillis();
	}
	
	/** 
	 * Finished the analyze. 
	 */
	public void setEndAnalyzeTime (){
		endanalyzeTime=System.currentTimeMillis();
		elapsedanalyzeTime+=endanalyzeTime-startanalyzeTime;
		analyzeTime=endanalyzeTime-startanalyzeTime;
		totalanalyzes++;
	}

	/** 
	 * Started to display. 
	 */
	public void setStartDisplayTime (){
		startdisplayTime=System.currentTimeMillis();
	}
	
	/** 
	 * Finished the display. 
	 */
	public void setEndDisplayTime (){
		enddisplayTime=System.currentTimeMillis();
		elapseddisplayTime+=enddisplayTime-startdisplayTime;
		displayTime=enddisplayTime-startdisplayTime;
		totaldisplays++;
	}
	
	/** 
	 * The statistics for the buffers. 
	 *  
	 * @return The statistics string.
	 */
	public String toString() {
		float FPS=0;
		/* Calculate the camera's grab rate. */
		if(totalgrabs>0) {
			avggrabTime=elapsedgrabTime/totalgrabs;
			elapsed=(System.currentTimeMillis()-firstGrabTime)/1000;
			if(elapsed>0)
				frameRate=totalgrabs/elapsed;

			if(ThisFPS>0) {
				FPS=1000/ThisFPS;
			}
		}
		
		/* Calculate the frame analyze rate. */
		if(totalanalyzes>0)			
			avganalyzeTime=elapsedanalyzeTime/totalanalyzes;
		/* Calculate the display rate. */
		if(totaldisplays>0) {
			avgdisplayTime=elapseddisplayTime/totaldisplays;
		}
		
		
		return("TIMESTAMP avgGrabTime="+avggrabTime+" totalGrabs="+totalgrabs+" "+ 
			"avgAnalyzeTime="+avganalyzeTime+" totalAnalyzes="+totalanalyzes+" "+  	
			 "avgDisplayTime="+avgdisplayTime+" totalDisplays="+totaldisplays+" "+ 
			  "grabTime="+grabTime+" "+
			   "analyzeTime="+analyzeTime+" "+
	  		    "displayTime="+displayTime+" "+
	  		     "avgFrameRate="+ frameRate+" "+
	  		       "frameRate="+FPS);
	}		
}


