package pinball.io;

import java.io.*;
import java.util.*;
import pinball.*;

/** 
 * Controls for the ADR2200 data IO device.
 */
public class ADR2200 {
	/** The serial port to write the data to. */
	private SerialIO serialIO;
	
	/** The test port. */
	private static final String testPort = "/dev/cuaa0";
	
	/** The value of the relays. */
	private int value=0;
	
	/** Which board to send the data to. */
	private int board=0;
	
	/** Is the device connected. 
	Set this to false, if you don't have this device, 
	but want to run the draw player. */
	private boolean deviceConnected=true;		
	
	/**
	 * Create the IO device at the port specified. 
	 *
	 * @param port serial port location.
	 */
	public ADR2200 (String port) {
		if(GameOptions.client())
			return;
		serialIO=new SerialIO(port);

		/* Configure the device. */
		try{setStates(0);}
		
		catch(Exception e){
			System.err.println("Problem setting up the ADR2200");
			System.exit(1);
		}
		
		System.err.println("ADR2200 made.");
		
	}
	
	/**
	 * Create the IO device for the port and board. 
	 *
	 * @param port serial port location.
	 * @param board which board if more than one board is daisy chained.
	 */
	public ADR2200 (String port, int board) {		
		if(GameOptions.client())
			return;
		this.board=board;
		serialIO=new SerialIO(port);

		/* Configure the device. */
		try{setStates(0);}
		
		catch(Exception e){
			System.err.println("Problem setting up the ADR2200");
			System.exit(1);
		}
		
		System.err.println("ADR2200 made.");
		
	}	
	
	/**
	 * The value of the relays
	 *
	 * @return the relay values.
	 */
	public synchronized int value() {
		return value;
	}
	
 	/**
 	 * Clear the output lines on the device.
	 *
	 * @param relay the relay to reset.
	 */
	public synchronized void resetOutputLine(int relay) throws IOException{		
		serialIO.write(new SerialData(board+"RK"+relay+"\r"));
	}

	/**
 	 * Set the output lines on the device.
	 *
	 * @param relay the relay to set.	
	 */
	public synchronized void setOutputLine(int relay) throws IOException{		
		serialIO.write(new SerialData(board+"SK"+relay+"\r"));		
	}

	/**
 	 * Set the output lines on the device.
	 *
	 * @param relays the states of the relays.
	 */
	public synchronized void setStates(int relays) throws IOException{
		value=0;		
		serialIO.write(new SerialData(board+"MK"+relays+"\r"));		
	}

	/**
 	 * Set the output lines on the device.
	 *
	 * @param relays the states of the relays.
	 */
	public synchronized void setStates(String relays) throws IOException{
		value=0;		
		serialIO.write(new SerialData(board+"SPK"+relays+"\r"));		
	}


	/** 
	 * Read the output lines. 
	 *
	 * @param input the line to check if it's on.
	 *
	 * @return the status of the line.
	 */
	public synchronized boolean ReadInput(int input) throws IOException{
		byte got;
		BitSet data=new BitSet(got=serialIO.read());
		System.err.println(got);
		return data.get(input);
	}
	
	/**
	 * Request a read from the output lines.
	 */
	public synchronized void requestRead() throws IOException {
		serialIO.write(new SerialData("board+PA\r"));
	}
	
	/**
	 * Clean out the data, so that all the devices are turned off.
	 */
	public synchronized void turnOff() {
		/* If the system does not have this device. */
		if(!deviceConnected)
			return;

		flushOutputData();
		try{
					
			System.err.println("Waiting for system to turn off the relays."); 
			for(int i=0;i<8;i++)
				resetOutputLine(i);
			/* Reset again for extra messure to prevent burnout. */
			for(int i=0;i<8;i++)
				resetOutputLine(i);
		}
		catch(Exception e){System.err.println("turnOff:"+e);}
	}


	/**
	 * Used to test the device. 
	 */	
	public static void main(String args[]) {
		new ADR2200(testPort).test();
	}
	
	/**
	 * Clear any data waiting to be read. 
	 */
	public synchronized void flushInputData() {
		serialIO.flushInputData();
	}

	/**
	 * Clear any commands waiting to go out. 
	 */
	public synchronized void flushOutputData() {
		serialIO.flushOutputData();
	}

	/**
	 * Test's the port.
	 */
	private void test() {
		board=0;
		turnOff();	
		board=1;
		/* Have the flippers constanting flipping in a pattern to test them. */
		while(true) {
			try { 
			/*
				setOutputLine(0);
				Thread.sleep(1000);
				setOutputLine(1);
				Thread.sleep(1000);
				setOutputLine(2);
				Thread.sleep(1000);
				resetOutputLine(0);
				Thread.sleep(1000);
				resetOutputLine(1);
				Thread.sleep(1000);
				resetOutputLine(2);
				Thread.sleep(1000);
				System.err.println("*");
			*/
			
			/*setOutputLine(5);
 			Thread.sleep(500);
			resetOutputLine(5);			
			//requestRead();
			Thread.sleep(500);
			board++;
			if(board>1)
				board=0;
			*/
			
			
			setOutputLine(2);
			Thread.sleep(500);
			turnOff();
			
			System.exit(0);
			//System.err.println(ReadInput(0));					
			}
		
			catch(Exception e){System.err.println(e);} 			
		}	

	}

	/**
	 * Generate random characters. 
	 */
	private byte random() {
		byte c = (byte)(Math.random()*25);
		c+=65;
		return c;
	}
} 
