package pinball.log;


import java.io.*;
import java.util.*;
import java.text.*;
import java.awt.*;

/**
 * This reads the log file and generates an html file with the statistics. 
 */
public class LogReport {
	/** File that data the data. */	
	private BufferedReader input;
	
	/** File that will get the data. */
	private PrintWriter output;
		
	/** Does the statistics on the ball. */
	private BallStats ballStats;
	
	/** Does the header report. */
	private HeaderReport header;
	
	/** Does the time stamp report. */
	private TimeStampReport timestamp;
	
	/** Only does the header once. */
	private boolean checkHeader=true;
	
	/** The log files that have been processed. */
	private Vector logFiles=new Vector();
	
	/** The area to view the video log. */
	private Viewer viewer;
	
	/** The last frame viewed. */
	private int lastFrame=0;
	
	/** The ball's location. */
	private int ballX=-1, ballY=-1;
	
	/** Finds the ball location. */
	private BallLocator ballLocator;
	
	/** Location of the ball found. */
	private Point ballLocation=new Point(-1,-1);
	
	/** Do this operation only once per frame. */
	private boolean oncePerFrame=true;
	
	/** Do this operation only once per game. */
	private boolean oncePerGame=true;
	
	/** Time for the last frame. Used to pause between the frames. */
	private long lastTime;
	
	/** Used to open the log files. */
	private LogFileAccess logFileAccess;
	
	/** Says if the display should be on. */
	private boolean display;

	/** Root directory. */
	private String root;
	
	/** Total frames. */
	private int totalFrames=0;
	
	/** Total left flips. */
	private int totalLeftFlips=0;
	
	/** Total right flips. */
	private int totalRightFlips=0;

	/**
	 * Creates the log reports.
	 *
	 * @param start The first log file to start with.
	 * @param out the file that will have the generated report.
	 * @param root is the root of the output.
	 * @param display says if the display should be turned on.	 
	 */
	public LogReport(String start, DataOutputStream out, String root, boolean display) {
		this.display=display;
		this.root=root;		
		
		output=new PrintWriter(out);
		
		header=new HeaderReport(output);
		timestamp=new TimeStampReport(output);

		ballLocator=new BallLocator();
		
		logFileAccess=new LogFileAccess(root,start,null);

		try{input=logFileAccess.getNext();}
		catch(Exception e) {input=null;}
		

		if(display) {
//			try {viewer=new Viewer(this);}
//			catch (Exception e){System.err.println(e);}
		}
		
		/* If the display is on, it has a button to start.
		   So this manually starts the processing. 
		 */
		if(!display)
			process();
			
	}
	
	
	/**
	 * Proces the log files. 
	 */
	public void process() {
		ballStats=new BallStats(output);
		
		begining();
		while(input!=null) {
			logFiles.addElement(logFileAccess.getLogFile());
			if(display)
				viewer.nextBallClip(logFileAccess.getLogName()+".dat");
			System.err.println("Processing log file: "+logFileAccess.getLogName());
			System.err.println();
			middle();
			checkHeader=false;
	
			try{input=logFileAccess.getNext();}
			catch(Exception e) {input=null;}
		}
		end();		
	}
	
	/** Write the begining of the html output file. */
	private void begining() {
		System.err.println("Begining to process");
		System.err.println();
		
		output.println("<HTML>\n<HEAD>\n\t<TITLE>Pinball Log File</TITLE>\n</HEAD>\n<BODY BGCOLOR=\"#ffffff\">");
	}
	
	/** Read the lines of the log file. */
	private void middle() {
		String data;
		try{	
			while((data=input.readLine())!=null){
				check(data);
			}
		}
		catch(IOException e) {System.err.println(e);}
		
 	}
	
	/** 
	 * Check the commands of the log file. 
	 *
	 * @param data the line from the log file.
	 */
	private void check(String data) {
		StringTokenizer tokens;
		String command;
		
		long time;
		int frame;
		String who;		

		try{
			tokens = new StringTokenizer(data,":");		
			command=tokens.nextToken();
		}
		catch(Exception e) {
			tokens=null;
			command="";
		}
		 		
		if(!command.equals("HEADER") && tokens!=null){
			tokens = new StringTokenizer(data,":");
			time=new Long(tokens.nextToken()).longValue();
			frame=new Integer(tokens.nextToken()).intValue();
			who=tokens.nextToken();
			
			if(oncePerFrame){			
				lastTime=time;
				totalFrames++;
			}
		
			command=tokens.nextToken(": ");						
		
			/* Set the ball's location to it's default location. */
			if(oncePerFrame) {
				ballLocation=new Point(-1,-1);
				oncePerFrame=false;
			}

			
			/* Do the stats commands. */
			if(command.equals("STATS"))
				ballStats.process(tokens);
			if(command.equals("TIMESTAMP"))
				timestamp.process(tokens);
			
			if(command.equals("BALL-LOCATION")) {
				ballLocation=ballLocator.findBallLocation(tokens);
			}

			/* Cound the left flips. */
			if(command.equals("FLIPLEFT")) {
				totalLeftFlips++;
			}
						
			/* Cound the right flips. */
			if(command.equals("FLIPRIGHT")) {
				totalRightFlips++;
			}
			
			
			
			/* Make sure only positive frames are displayed. */				
			if(frame>lastFrame) {
				oncePerFrame=true;
				if(display) {
					viewer.readFrame(ballLocation.x,ballLocation.y,"",0);
   					/* Pause between frames. */
					try{Thread.sleep(time-lastTime);}
					catch(Exception e){}
				}
				lastFrame=frame;				
				lastTime=time;


			}																

		}
		
		if(checkHeader && command.equals("HEADER")){
			/* I don't need this token, it's just the name of the object who wrote the header. */
			tokens.nextToken(" ");

			header.parse(tokens.nextToken());

		}
	}	
	
	/** Flipper efficiency. */
	private void flipperEfficiency() {
		float leftEfficiency=((float)totalLeftFlips/totalFrames)*100;
		float rightEfficiency=((float)totalRightFlips/totalFrames)*100;
		float totalEfficiency=((float)(totalLeftFlips+totalRightFlips)/totalFrames)*100;
		output.println("<HR>Percentage of frames spent flipping left flipper: "+leftEfficiency);
		output.println("<HR>Percentage of frames spent flipping right flipper: "+rightEfficiency);
		output.println("<HR>Percentage of frames overall spent flipping flippers: "+totalEfficiency);	
		output.println("<HR>Total flips:"+(totalLeftFlips+totalRightFlips)+" total frames:"+totalFrames);	
		
	}	
	
	/** Write the end of the log file, and close it up. */	
	private void end() {
		header.writeInfo();
		ballStats.writeInfo();
		timestamp.writeInfo();
		
		flipperEfficiency();
		
		new FrameSet(logFiles,root);
		
		/* List the log files used. */
		output.println("<HR>Log files used...<P>");
		String name;
		for(int i=0;i<logFiles.size();i++) {
			name=(String)logFiles.elementAt(i);
			output.println(name+"<DT>");
		}
		
		output.println("<HR><A HREF=\"http://www.sci.brooklyn.cuny.edu/~pinball/\" target=\"_top\">The Pinball Player Project</A><P></BODY></HTML>");
		output.flush();
		output.close();

		System.err.println("Finished analyzing.");
		System.exit(0);
	}
	

}
