package pinball.log;

import pinball.*;

import java.io.*;
import java.util.*;
import java.text.*;
import java.awt.*;

/**
 * This reads the log file and generates an html file with the statistics. 
 */
public class MovieViewer extends Thread{
	/** File that data the data. */	
	private BufferedReader input;
			
	/** Does the statistics on the ball. */
	private BallStats ballStats;
		
	/** Only does the header once. */
	private boolean checkHeader=true;
	
	/** The log files that have been processed. */
	private Vector logFiles=new Vector();
	
	/** The area to view the video log. */
	private Viewer viewer;
	
	/** The last frame viewed. */
	private int lastFrame=0;
	
	/** The ball's location. */
	private int ballX=-1, ballY=-1;
	
	/** Finds the ball location. */
	private BallLocator ballLocator;
	
	/** Location of the ball found. */
	private Point ballLocation=new Point(-1,-1);
	
	/** Do this operation only once per frame. */
	private boolean oncePerFrame=true;
	
	/** Do this operation only once per game. */
	private boolean oncePerGame=true;
	
	/** Time for the last frame. Used to pause between the frames. */
	private long lastTime;
	
	/** Used to open the log files. */
	private LogFileAccess logFileAccess;
	
	/** Says if the display should be on. */
	private boolean display;

	/** Display only one clip. */
	private boolean single;
	
	/** Text from the log, odd frames. */
	private String oddText=null;

	/** Text from the log, even frames. */
	private String evenText=null;
	
	/** The frame info for all the frames. */
	private Vector textData=new Vector();

	/** The maximum size of the buffer. */
	private int maxBuffer=GameOptions.bufferSize();
	
	/** Locations of the flippers. */
	private int leftX,leftY,rightX,rightY;

	/**
	 * Creates the log reports.
	 *
	 * @param start The first log file to start with.
	 * @param player the applet.
	 * @param single only one movie file.
	 * @param xsize the width of the image.
	 * @param xsize the height of the image.
	 */
	public MovieViewer(String start, MoviePlayer player, boolean single, int xsize, int ysize) {
		this.single=single;
		ballLocator=new BallLocator();
		
		logFileAccess=new LogFileAccess("",start,player);

		try{input=logFileAccess.getNext();}
		catch(Exception e) {input=null;}
		

		try {viewer=new Viewer(this,player,xsize,ysize);}
		catch (Exception e){System.err.println(e);System.exit(99);}
		
	}

	/** Get the viewer. */	
	public Viewer viewer() {
		return viewer;
	}
	
	/** Startup the viewing. */
	public void run() {
		process();
	}
	
	/**
	 * Proces the log files. 
	 */
	public void process() {
		System.err.println("Displaying movie");
		while(input!=null) { 
			logFiles.addElement(logFileAccess.getLogName());
			viewer.nextBallClip(logFileAccess.getLogName()+".dat");
			//viewer.nextBallClip(logFileAccess.getLogName()+".zip");
			System.err.println("Processing log file: "+logFileAccess.getLogName());
			System.err.println();
			middle();
			checkHeader=false;
	
			/* If playing more than one clip continue, otherwise stop. */
			if(!single) {			
				try{input=logFileAccess.getNext();}
				catch(Exception e) {input=null;}
			}
			else break;
			
		}
		System.err.println("DONE READING IN CLIP(S)");
		viewer.setText("Finished Reading.");
		playMovie();				
	}
			
	/** Read the lines of the log file. */
	private void middle() {
		String data=null;
		try{	
			while((data=input.readLine())!=null)
				textData.addElement(new String(data));
		}
		catch(IOException e) {System.err.println(e);}
		System.err.println("Read in log file.");
		
		int currentFrame=1;
		boolean found=true;
		boolean reading=true;
		boolean firstRead=true;
		while(found) {
			found=false;
			for(int i=0;i<textData.size();i++) {
				data=(String) textData.elementAt(i);
				int thisFrame=whichFrame(data);
				if(firstRead && thisFrame!=0)  {
					/* Since the first frame in the log file */
					/* is a grabbed frame, previous buffers  */
					/* are ignored. 			 */
					 
					currentFrame=thisFrame;
					firstRead=false;
					lastFrame=thisFrame;
				}
				if(thisFrame==currentFrame) {
					check(data);
					textData.removeElementAt(i);
					/* When the data is removed the elements shift down. */
					i--;
					found=true;					
				}
				
				else if(thisFrame>(currentFrame+maxBuffer))
					break;
			}
			System.err.println("Frame "+currentFrame);			
			currentFrame++;			
		}
		System.err.println("Done analyzing");	

		
 	}
	
	/** 
	 * Find the frame number.
	 *
	 * @param data the line from the log file.
	 */
	private int whichFrame(String data) {

		StringTokenizer tokens;
		String command;
		
		long time;
		int frame;
		String who;		
		String text="";
		
		tokens = new StringTokenizer(data,":");
		
		command=tokens.nextToken();
		
		 		
		if(command.equals("HEADER")){
			String message;

			tokens = new StringTokenizer(data,":");
			who=tokens.nextToken();
			message=tokens.nextToken(": ");
			message=tokens.nextToken(": ");
			tokens = new StringTokenizer(message,"=");
			String item=tokens.nextToken();
			if(item.equals("bufferSize")) {
				int buffers=new Integer(tokens.nextToken()).intValue();
				maxBuffer=buffers;
			}

			if(item.equals("leftFlipperX")) {
				leftX=new Integer(tokens.nextToken()).intValue();
			}

			if(item.equals("leftFlipperY")) {
				leftY=new Integer(tokens.nextToken()).intValue();
			}

			if(item.equals("rightFlipperX")) {
				rightX=new Integer(tokens.nextToken()).intValue();
			}

			if(item.equals("rightFlipperY")) {
				rightY=new Integer(tokens.nextToken()).intValue();
				viewer.setFlippers(new Point(leftX,leftY),new Point(rightX,rightY));
			}
		}

		if(!command.equals("HEADER")){
			tokens = new StringTokenizer(data,":");
			time=new Long(tokens.nextToken()).longValue();
			frame=new Integer(tokens.nextToken()).intValue();
			who=tokens.nextToken();
			return frame;
		}
		
		else return 0;	
	}
	
	/** 
	 * Check the commands of the log file. 
	 *
	 * @param data the line from the log file.
	 */
	private void check(String data) {
		StringTokenizer tokens;
		String command;
		
		long time;
		long sleepTime=0;
		int frame;
		String who;		
		String text="";
				
		tokens = new StringTokenizer(data,":");
		
		command=tokens.nextToken();	
		 		
		if(!command.equals("HEADER")){
			tokens = new StringTokenizer(data,":");
			time=new Long(tokens.nextToken()).longValue();
			frame=new Integer(tokens.nextToken()).intValue();
			who=tokens.nextToken();					
				
		
			command=tokens.nextToken(": ");			

			/* Even lines of text. */
			if(frame%2==0) {
				if(evenText==null)
					evenText=data;
				else evenText+="\n"+data;
			}

			/* Odd lines of text. */								
			else {
				if(oddText==null)
					oddText=data;
				else oddText+="\n"+data;
				
			}

			/* Set the ball's location to it's default location. */
			if(command.equals("FRAME_GRABBED")) {				
				sleepTime=time-lastTime;
				if(sleepTime>1000)
					sleepTime=100;
				lastTime=time;
				oncePerFrame=false;
			}

			
			/* Get the ball's location. */
			if(command.equals("BALL-LOCATION")) {
				ballLocation=ballLocator.findBallLocation(tokens);
			}
			
			/* Make sure only positive frames are displayed. */
			if(frame>lastFrame) {
				if(lastFrame%2==0 && evenText!=null){
					text=evenText;
					evenText=null;
				}
				else if(oddText!=null){
					text=oddText;
					oddText=null;
				}
				viewer.readFrame(ballLocation.x,ballLocation.y,text,sleepTime);
				oncePerFrame=true;
				
				lastTime=time;		
				ballLocation=new Point(-1,-1);						
			}
			lastFrame=frame;																

		}
	}	
	
	/** Play the movie. */	
	public void playMovie() {
		while(true) {
			try{	while(viewer.paused())
					Thread.sleep(100);			
			}
			catch(Exception e){}
			if(viewer.play())
				viewer.nextFrame();
			if(viewer.rewind())
				viewer.previousFrame();
			try{Thread.yield();}
			catch(Exception e){}
		}
	}	
}
