package pinball.log;

import pinball.*;
import pinball.buffer.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/**
 * Displays the video of the log file. 
 * 
 */
public class Viewer{
	/** Canvas to draw to. */
	private ViewerCanvas canvas;
	
	/** Frame to display the video. */
	private CameraFrame frame;

	/** Input file. */
        private ObjectInputStream imageFile;
	
	/** Video log file. */
	private VideoLog videoLog;
	
	/** The button that starts the viewer. */
	private Button go=new Button("GO");
	
	/** Ball's location. */
	private int x=-1, y=-1;
	
	/** Reads the statistics log files. */
	private MovieViewer log;

	/** Is the system running. */
	private boolean running=false;
	
	
	/** Applet to display the image. */
	private MoviePlayer player;
	
	/** The size of the applet. */
	private Dimension d;
	
	/** Text from the log. */
	private TextArea logText;
	
	/** The viewer is paused. */
	private boolean paused=true;
	
	/** Step though a single frame. */
	private boolean step=false;
	
	/** Saved frames. */
	private Vector frames=new Vector();
	
	/** The latest frame to be read in. */
	private int lastFrameRead=0;

	/** The latest frame to be displayed. */
	private int lastFrameDisplayed=0;
	
	/** Is the system rewinding. */
	private boolean rewind=false;
	
	/** Is the system playing. */
	private boolean play=true;

	/** Is the system playing fast. */	
	private boolean fast=false;

	/**
	 * Sets up the video viewer.
	 *
	 * @param log The file analyzer.
	 *
	 */
	public Viewer(MovieViewer log, MoviePlayer player, int xsize, int ysize) {
		d=player.size();
		this.log=log;
		this.player=player;
		videoLog=new VideoLog(player);
		
		startup(xsize, ysize);
		canvas.makeImage();
		
	}
		
	/** Sets up the layout. */
	private void startup(int xsize, int ysize) {
		//player.resize(xsize,ysize);
		//player.setLayout(new FlowLayout(FlowLayout.CENTER));
		player.setLayout(new BorderLayout());
		canvas=new ViewerCanvas(xsize*2,ysize*2);
		Panel image=new Panel();
		image.setLayout(new FlowLayout(FlowLayout.CENTER));		
		image.add(canvas);		
		player.add("Center",image);
		Panel stuff=new Panel();
		stuff.setLayout(new FlowLayout(FlowLayout.CENTER));
		stuff.add(logText=new TextArea(6,50));		
		player.add("South", stuff);
		logText.setText("LOADING LOG FILE. PLEASE WAIT.");
	}

	/**
	 * Read a frame from the log file. 
	 *
	 * @param x the x location of the ball.
	 * @param y the y location of the ball.	 
	 * @param text the text of the frame.	 	 
 	 * @param sleepTime the time to pause between frames.
	 */ 
	public synchronized void readFrame(int x, int y, String text,long sleepTime) {
		boolean hasData=true;
		CameraFrame frame=null;
		SavedFrame savedFrame;
		
		/* Read a frame from the log file and display it on the screen. */
		try{frame=(CameraFrame)imageFile.readObject();}
		catch(IOException e){hasData=false;}
		catch (ClassNotFoundException e){hasData=false;}			
		if(hasData) {
			lastFrameRead++;
			frames.addElement(new SavedFrame(frame,text,x,y,sleepTime));
			savedFrame=(SavedFrame)frames.elementAt(lastFrameDisplayed);
			if(lastFrameRead==1) {
				displayFrame(savedFrame);
			}
		}
	}

	/**
	 * Display the frame.
	 *
	 * @param frame the frame from the camera.
	 * @param text the text to be displayed.
	 * @param x the x location of the ball.
	 * @param y the y location of the ball.
	 */
	public void displayFrame(CameraFrame frame, String text, int x, int y){
		canvas.updateFrame(frame, x, y);
		if(text!=null)
			logText.setText(text);	
		else if (text==null) logText.setText(""); 
		if(step){
			step=false;
			paused=true;
		}		
	}
	/**
	 * Displays a frame that was read in. 
	 *
	 * @param frame the frame read in.
	 */
	public void displayFrame(SavedFrame frame) {
		displayFrame(frame.frame(),frame.text(),frame.x(),frame.y());
	}
	
	/**
	 * Set the text to be displayed.
	 *
	 * @param text the text for the frame. 
	 */
	public void setText(String text) {
		logText.setText(text);		
	}
	
	/** 
	 * Get the next ball's video clip.
	 *
	 * @param clip the file name of the next video clip.
	 */
	public void nextBallClip(String clip) {
		System.err.println("Opening video clip: "+clip);
		try{
		imageFile=videoLog.getReadNext(clip);
		}
		catch(Exception e){System.err.println(e);}
	}
	
	/*
	 * Set the paused condition. 
	 * 
	 * @param p the paused condition.
	 */	 
	public void paused(boolean p) {
		paused=p;
	}
	
	/** 
	 * Is the viewer paused. 
	 *
	 * @return the paused condition.
	 */
	public boolean paused() {
		return paused;
	}
	
	/**
	 * Display the next frame.
	 */
	public void nextFrame() {
		SavedFrame frame;

		if(lastFrameDisplayed<lastFrameRead-1) {
//			System.err.println("foward:"+lastFrameDisplayed);
			lastFrameDisplayed++;
	 		frame=(SavedFrame)frames.elementAt(lastFrameDisplayed);
			displayFrame(frame);
			if(!fast) {
				try{Thread.sleep(frame.sleepTime());}
				catch(Exception e){}
			}
		}
	}	
	
	/**
	 * Display the previous frame.
	 */	
	public void previousFrame() {
		SavedFrame frame;		
		if(lastFrameDisplayed>0) {
//			System.err.println("back:"+lastFrameDisplayed);
			lastFrameDisplayed--;
			frame=(SavedFrame)frames.elementAt(lastFrameDisplayed);
			displayFrame(frame);
			if(!fast) {
				try{Thread.sleep(frame.sleepTime());}
				catch(Exception e){}
			}
		}
	}

	/**
	 * Go to the next frame.
	 */	
	public void step() {
		step=true;
		nextFrame();
		step=false;
		//paused=false;
	}	
	
	/**
	 * Go to the previous frame.
	 */	
	public void stepBack() {
		SavedFrame frame;
		step=true;	
		previousFrame();
		step=false;		
		//paused=false;
	}	

	/**
	 * The system will rewind.
	 */		
	public void rewind(boolean r) {
		rewind=r;		
	}

	/**
	 * Rewind the video.
	 */		
	public void begining() {
		SavedFrame frame;
		lastFrameDisplayed=0;
 		frame=(SavedFrame)frames.elementAt(lastFrameDisplayed);
		displayFrame(frame);
 	}

	/**
	 * Set the play condition.
	 *
	 * @param play the play condition.
	 */		
	public void play(boolean p) {
		play=p;
	}

	/**
	 * Should the system rewind the video.
	 *
	 * @return the rewind status.
	 */			
	public boolean rewind() {
		return rewind;
	}
	
	/**
	 * Is the system playing?
	 *
	 * @return play status.
	 */			
	public boolean play() {
		return play;
	}	
	
	/**
	 * Set fast mode.
	 *
	 * @param f fast status.
	 */			
	public void fast(boolean f) {
		fast=f;
	}
	
	/**
	 * Is the system playing fast?
	 */				
	public boolean fast() {
		return fast;
	}
	
	/**
	 * Set the locations of the flippers.
	 *
	 * @param left left flipper's location.
	 * @param right right flipper's location.	 
	 */				
	public void setFlippers(Point left, Point right) {
		canvas.setFlippers(left,right);
	}
}
