package pinball.part;

import java.io.*;
import java.awt.*;
import java.util.*;

import pinball.buffer.*;
import pinball.*;
import pinball.log.*;

/**
 * The camera is what allows the system to see. It grabs a frame and sends it
 * into the computer. The camera needs various parameters, such as size,
 * brightness, and contrast, to know how to take a picture.
 */
public class Camera implements Serializable{
	/* Used to keep the same serialized version. */
	static final long serialVersionUID = 6690706735923851049L;

	/** horizontal size */	
	protected int xsize=0;           
	
	/** vertical size */
	protected int ysize=0;           

	/** horizontal image offset */
	protected int xoffset=0;          
	
	/** vertical image offset */
	protected int yoffset=0;           
	
	/** The camera's transfer. */	
	protected int transfer;		
	
	/** Zoom */   
	protected int zoom;		 
	
	/** Amount of display buffers. */
	protected int buffers;		  
	
	/** Display every nth frame. */
	protected int displayRate;	  
 	
	/** Brightness of the camera. */
	protected int brightness=-1;
	
	/** Contrast of the camera. */
	protected int contrast=-1;	
	
	/** Contrast of the camera. */
	protected int transferset;			
    
	/** Max width of the frame. */
	public static final int maxWidth = 320;
	
	/** Max height of the frame. */	
	public static final int maxHeight = 240;


	/** Output file. */
        protected ObjectOutputStream logout;
	
	/** Input file. */
        protected ObjectInputStream login;	

	/** Video log, for recording or playing a game. */
	protected VideoLog videoLog;
	
	/* How the camera is to get pictures. */
	
	/** Live from the camera. */
	protected boolean live=false;
	
	/** Recording to a file. */
	protected boolean record=false;
	
	/** Playing from a file. */
	protected boolean play=false;
	
	/** Current frame from the camera. */
	protected CameraFrame cameraFrame;	
	
	/* Load the library. */
	static {
		System.err.println("Starting Camera.");
		System.loadLibrary("Camera");
	}

	/**
	 * Setup the camera.
	 *
	 * @param driver load the driver or not.
	 */
	public Camera(boolean driver)  {
		if(driver)
			driver();
			
		System.err.println("Width "+xsize+" Height "+ysize);
	}
	
	/**
	 * Setup the camera information. 
	 */
	public void cameraInfo() {

		System.err.println("Camera Recordable created.");
		
		/* If recording. */
		if(GameOptions.record()) {
			System.err.println("Recording");

			record=true;
			videoLog=new VideoLog();
			String logFile=PinballLog.videoLogFileName();
			try{
				logout=videoLog.getWriteNext(logFile);
			}
			catch(Exception e){
				System.err.println("Cannot create "+logFile);
				System.exit(1);
			}
			System.err.println("Video log: "+logFile);
		}
		
		/* If playing. */
		else if(GameOptions.play()) {
			System.err.println("Playing");		
			play=true;
			
			String logFile=GameOptions.logFileName()+".dat";
			try{
			videoLog=new VideoLog(logFile);
			
				login=videoLog.getReadNext();
				cameraFrame=(CameraFrame)login.readObject();
				xsize=cameraFrame.getWidth();
				ysize=cameraFrame.getHeight();
				xoffset=cameraFrame.getXOffset();
				yoffset=cameraFrame.getYOffset();
				transferset=cameraFrame.getTransfer();
				
			}
			catch(Exception e){
				System.err.println(logFile+" not found");
				System.exit(1);
			}
			
		}
		
		/* Live. */
		else {
			live=true;
		}
		
	}

	/**
	 * Get the name of the camera. 
	 * 
	 * @return camera's name.
	 */	
	public String toString() {
		return "CameraRecordable is correct";
	}
	
	/**
	 * Grab a frame from the camera.
	 *
	 * @param buffinfo the buffer to store the frame in. 
	 */
	public void grabFrame(BufferInfo buffinfo) {
		if(live)
			GrabFrame(buffinfo.get());
		
		/* Record the image. */
		else if(record) {			
			GrabFrame(buffinfo.get());
			try { 
				logout.reset();
				logout.writeObject(buffinfo.getCameraFrame());
				logout.flush();
			}
			catch (IOException e){System.err.println(e);}
			try { logout.flush();}
			catch (IOException e){System.err.println(e);}
			
		}
		
		/* Get frame from the file. */
		else if(play) {
			String logFile="";
			try { 
				buffinfo.set(cameraFrame);
				cameraFrame=(CameraFrame)login.readObject();
			}
			/* Log file is finished, go for the next file. */
			catch (IOException e){
				System.err.println("Going to the next video file");				
				try{
					login=videoLog.getReadNext();
				}
				catch(Exception f){
					System.err.println("Completed reading file.");
					System.exit(0);
				}
			}
			catch (ClassNotFoundException e){System.err.println(e);}			
			
		}

		else {
			System.err.println("Something's not right. System is neither in play nor record mode.");
			System.exit(1);
		}		
	
	}
	
	/**
	 * Go to the next log video log file. 
	 */
	public synchronized void nextBall(){
		/* If recording, go to the next log file . */
		if(record) {
			String logFile=PinballLog.videoLogFileName();
			try{
				logout=videoLog.getWriteNext(logFile);
			}
			catch(Exception e){
				System.err.println("Cannot create "+logFile);
				System.exit(1);
			}
			System.err.println("Video log: "+logFile);
		}
	
	}

	/**
	 * Load the driver.
	 */	
	private void driver() {
		/* Setup some overwrite options. */
		Point newSize=GameOptions.getCameraSize(), newOffset=GameOptions.getCameraOffset();

		xsize=newSize.x;
		ysize=newSize.y;
		
		xoffset=newOffset.x;		
		yoffset=newOffset.y;
		
		transfer=GameOptions.getCameraTransfer();
			
		Init();
	}
	
		
	/** Initalize the camera. */
	protected native void Init();
	
	/**
	 * Grab the frame from the camera. 
	 *
	 * @param buffer the location to store the frame. 
	 */
	public native void GrabFrame(byte []buffer);
		
	
	/**
	 * Resize the frame. 
	 * 
	 * @param width the width of the frame.
	 * @param height the height of the frame.
	 * @param xoffset the xoffset of the frame.	 
	 * @param yoffset the yoffset of the frame.	 
	 */
	protected native void FrameResize(int width, int hight, int xoffset, int offset, int transfer);
	
	
	/**
	 * Get the frame's x size.
	 *
	 * @return x size.
	 */
	public int xSize(){
		return xsize;
	}

	/**
	 * Get the frame's y size.
	 *
	 * @return y size.
	 */
	public int ySize(){
		return ysize;
	}	
	
	/**
	 * Get the number of bytes for the frame. 
	 *
	 * @return number of bytes for this frame.
	 */
	public int frameSize() {
		return xsize*ysize;
	}
	
	/**
	 * Get the frame's zoom size. 
	 *
	 * @return zoom size.
	 */
	public int zoomSize() {
		return zoom;
	}
	
	/**
	 * Get the frame's x offset. 
	 *
	 * @return x offset. 
	 */
	public int XOffset() {
		return xoffset;
	}

	/**
	 * Get the frame's y offset. 
	 *
	 * @return y offset. 
	 */	
	public int YOffset() {
		return yoffset;
	}
	
	/**
	 * Resize the frame. 
	 *
	 * @param buffinfo with frame size to resize.
	 */
	public void resizeFrame(BufferInfo buffinfo) {
		xsize=buffinfo.resizeX();
		ysize=buffinfo.resizeY();
		xoffset=buffinfo.resizeXOffset();
		yoffset=buffinfo.resizeYOffset();
		transfer=buffinfo.resizeTransfer();
		
		/* Check image for proper size. */
		if(xsize<0 || ysize<0 || xoffset<0 || yoffset<0) {
			System.err.println("Mistake in image size.");
			return;
		}	
		
		FrameResize(xsize*transfer,ysize*transfer,xoffset*transfer,yoffset*transfer,transfer);
		
	}

	/**
	 * Get the transfer size. 
	 *
	 * @return transfer size.
	 */
	public int transfer() {
		return transfer;
	}
	
	/**
	 * Get brightness.
	 *
	 * @return brightness.
	 */	 
	public int brightness() {
		return brightness;
	}
	
	/**
	 * Get contrast.
	 *
	 * @return contrast.
	 */	 
	public int contrast() {
		return contrast;
	}

	/**
	 * Image dimensions
	 *
	 * @return image dimensions.
	 */
	public Dimension imageSize(){
		return new Dimension(xsize,ysize);
	}
}
