package pinball.player;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/**
 * Draws the frame from the camera on the screen.
 * It also draws various lines to indicate different zones on the screen.
 * As well, it draws the locations of the ball and the flippers.
 */
public class DrawingCanvas extends Canvas{

	/** The greyscale values for the painting of the image.*/
	protected	ColorModel model;
	
	/** The x size of the canvas (image). */
	protected	int width;
	
	/** The y size of the canvas (image). */	
	protected	int height;

	/** The current information buffer. */
	protected Buffer buffer;	 
	
	/** The information in the buffer. */
	protected BufferInfo buffinfo;
		
	/** Used when the flippers are found. */
	
	/** Yoffset of image. */
	protected int aty;	
	
	/** Xoffset of image. */
	protected int atx;	
	
	/** Used to draw the image to. */
	
	/** Where to draw to off screen.*/
	protected Graphics graphicsContext;
	
	/** The image to draw to. */
	protected Image  frameBuffer;
	

	/** The stub which contains various lines from the strategy to be drawn
	to the screen. Used to show how the strategy chooses to flip. */
	protected Stub stub;
	
	/** 
	 * Initalize the DrawingBoard. 
	 *
	 * @param w width of the image.
	 * @param h height of the image.
	 */
	public DrawingCanvas(int w, int h){
		width=w;
		height=h;		
		
		System.err.println("Canvas size: "+width+" "+height);
		
	       /* Create the image's color table. (Grey scale)*/
        	byte grey[] = new byte[64];
	        for (int i = 0; i < 64; i++) {
        	   grey[i] = (byte) (i*4);
	        }
		
		/* Pure white. */
		grey[63]= (byte) 255;
	      	model = new IndexColorModel(6, 64, grey, grey, grey);				
	      	
	}

	/**
	 * Used in the extended classed. 
	 */
	public DrawingCanvas() {}

	/**
	 * Make the off screen image to draw to. 
	 */
	public void makeImage() {
		/* Create frameBuffer. */
	        frameBuffer = createImage(width,height);

		/* Create off screen graphics context. */
		graphicsContext = frameBuffer.getGraphics();
	}


	/** 
	 * Setup the stub from the system.
	 * 
	 * @param stub the stub from the system.
	 */
	 public void setStub(Stub stub) {
	 	this.stub=stub;
		if(stub==null)
			System.err.println("***Null Stub***");	
	}

	/**
	 * Update the image's data. Draw to the off screen image.
	 *
	 * @param buffinfo the buffer information that has the image.
	 */
	public void updateFrame (BufferInfo buffinfo) {		
		MemoryImageSource mis;
		/* The location of the ball. */
		int x=buffinfo.blip().x();
		int y=buffinfo.blip().y();
		
		if(GameOptions.verbose() && buffinfo.verboseFrame()!=null) {
			/* Make the image.*/
		        mis = new MemoryImageSource(buffinfo.width(),buffinfo.height(),
  		   		   		model, buffinfo.verboseFrame(), 0, buffinfo.width());
		}
		
		else 
		/* Make the image.*/
	        mis = new MemoryImageSource(buffinfo.width(),buffinfo.height(),
  		   	   		model, buffinfo.get(), 0, buffinfo.width());

	        /* The image from the camera. */
		Image myImage = createImage(mis);

		if(GameOptions.debug()>=2)
	     		  System.err.println("Frame Updated.");
		
		/* Draw new image in the frameBuffer. */

		/* Clear the background if the buffer calls for it. 
		   Used when resizing to erase the background.*/
		if(buffinfo.clearing()) {
			Dimension d=getSize();
			graphicsContext.clearRect(0,0,d.width,d.height);
			buffinfo.clearing(false);
		}

		/* Draw the image of the game. */
		graphicsContext.drawImage(myImage,buffinfo.XOffset(),buffinfo.YOffset(),buffinfo.width(),buffinfo.height(),this);
			
		/* Draw the flipper zone. */	
//		if(GameOptions.flipperZone()!=-1) {	
//			graphicsContext.setColor(Color.yellow);
//			graphicsContext.drawRect(buffinfo.XOffset(),GameOptions.flipperZone(),buffinfo.width()-1,GameOptions.flipperZoneSize());
//		}		
		
		/* Have the strategy draw it's lines so you can see the decision process. */
		if(stub!=null)
			stub.strategyDraw(graphicsContext,buffinfo.width(),buffinfo.height());		
		
		/* Draw the cross for the ball's location. */
		if(x>-1 && y>-1) {
			x+=buffinfo.XOffset();
			y+=buffinfo.YOffset();

			if(GameOptions.verbose())
				graphicsContext.setColor(Color.blue);
			else graphicsContext.setColor(Color.yellow);
			graphicsContext.drawLine(x,y-5,x,y+5);
			graphicsContext.drawLine(x+5,y,x-5,y);
		}

		paint(getGraphics());	
	}


	/**
	 * Draw the images. 
	 * 
  	 * @param g Where to draw to.
	 */
	public void paint(Graphics g) {        	
		if(GameOptions.debug()>=2)
	        	System.err.println("Drawing a frame.\n");        		        	        	
	                	               					
        	/* display the image on the screen. */
		if(frameBuffer!=null)
			g.drawImage(frameBuffer,0,0,width,height,this);
        }

	/**
	 * Called when the window is updated. 
  	 * @param g Where to draw to.
	 */
        public void update(Graphics g) {
	
		paint(g);

	}
	
	
	/** 
	 * Method used in creating the canvas. Sets up the size.
	 *
	 * @return the minimum size.
	 */
	public Dimension getPreferredSize(){
		return getMinimumSize();
	}
	
  	/** 
	 * Method used in creating the canvas. Sets up the size.
	 * @return the dimension for the canvas.
	 */
	public Dimension getMinimumSize() {
		return new Dimension(width,height);
	}
}
