package pinball.player;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;


/**
 * This keeps track of the buffer that are to be used by the strategy thread.
 */
class StrategyQueue {
	/** Buffers waiting to be used. */
	private Vector queue=new Vector();	
	
	/** Buffers waiting to be released. */
	private Vector release=new Vector();
		
	/** Stub from strategy. Used for logging purposes. */
	private Stub stub;

	/**
	 * Make a new queue.
	 * 
	 * @param s stub to use.
	 */
	public StrategyQueue(Stub s) {
		stub=s;
	}
	
	/**
	 * For extended classes.
	 */
	public StrategyQueue(){}

	/**
	 * Get the name of the object.
	 * 
	 * @param name of the object.
	 */
	public String toString() {
		return "StrategyQueue";
	}


	/**
	 * Add a buffer to the queue.
	 *
	 * @param b the buffer to add.
	 */
	public synchronized void add(Buffer b) {
//		System.err.println("Add Size "+queue.size());
		queue.addElement(b);
	}

	/**
	 * Get the next buffer in the queue.
	 *
	 * @param the next buffer in the queue.
	 */
	public synchronized Buffer next() {
//		System.err.println("REMOVING");
		
		Buffer b=(Buffer)queue.elementAt(0);
		release.addElement(b);
		queue.removeElementAt(0);
		return b;
	}
	
	/**
	 * Get the recent buffer in the queue.
	 *
	 * @param the recent buffer in the queue.
	 */
	public synchronized Buffer recent() {
		Buffer b;
		int size=queue.size();
				
//		System.err.println("Remove Size "+queue.size());  
		
		if(size==0)
			return null;  
		
		for(int at=0;at<(size-1);at++) {
			b=(Buffer)queue.elementAt(0);
			queue.removeElementAt(0);
			//System.err.println("Strategy Skipping "+b.id());			
			b.lock().strategied(b.id());
		}
		
		b=(Buffer)queue.elementAt(0);
		queue.removeElementAt(0);
		release.addElement(b);
		
//		System.err.println("BLIP "+b.info().blip().y());

		return b;
	}
	
	/**
	 * Checks if the queue is empty.
	 *
	 * @return empty status.
	 */
	public synchronized boolean empty() {
		if(queue.size()!=0)
			return false;
		return true;
	}
	
	/**
	 * Release a buffer from the queue. It also releases any unreleased
	 * buffers waiting to be released that might not have been released.
	 *
	 * @param buff the buffer to be released.
	 */
	public synchronized void release(int buff) {
		Buffer b=null;
		int size=release.size();
						
//		System.err.println("Release Size "+size);
		
		if(size==0)
			return;  
		
		for(int at=0;at<=(size-1);at++) {
			b=(Buffer)release.elementAt(0);
						
			release.removeElementAt(0);
//			System.err.println("Releasing "+b.id());
			b.lock().strategied(b.id());						
			
			if(b.id()==buff) {
				b.strategyUnlock("Natural");
				stub.logit(b.info().number(),"naturalStrategyUnlock="+b.id(),1,this);
				break;
			}
			
			else if(b.id()!=buff) {
				b.strategyUnlock("Forced");
				stub.logit(b.info().number(),"forcedStrategyUnlock="+b.id(),1,this);
			}
		}				

	}
	
	
	/**
	 * Release all the buffers from the queue.
	 */
	public synchronized void releaseAll() {
		Buffer b=null;
		int size=release.size();
						
		if(size==0)
			return;  
		
		for(int at=0;at<=(size-1);at++) {
			b=(Buffer)release.elementAt(0);
						
			release.removeElementAt(0);
			b.lock().strategied(b.id());						
		}				

	}

}
