package pinball.player.adjustable;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/** 
 * The flipper's zone can be adjusted by the user. 
 * Clicking the mouse on the screen, the user can define the zone for the left
 * and right flippers. When the ball goes into the flipper's zone, that flipper
 * flips.
 */
public class AdjustableFlippers extends Adjustments {
	/** Left flipper. */
	private final int LEFT=0;
	
	/** Right flipper. */
	private final int RIGHT=1;
	
	/** Which flipper is being edited. */
	private char which=LEFT;
	
	/** The flipper's flip zones. */
	private Polygon flippers[]=new Polygon[2];
	
	/** Who's the canvas to draw on. */
	private AdjustableDrawingCanvas parent; 		
	
	
	/** Output file, for flipper locations. */
	private FileOutputStream out=null;
	/** Output file, for flipper locations. */
          private ObjectOutputStream logout=null;

	/** Input file, for flipper locations. */
 	private FileInputStream in=null;
	/** Input file, for flipper locations. */
          private ObjectInputStream login=null;
	
	/** File name of the flipper zones. */
	private String polygonFilename="";
	
	/** Was the file found. */
	private boolean fileFound=false;

	/* Create the adjustable flippers. */
	public AdjustableFlippers () {
		
		String[] args=GameOptions.getArgs();
		
	        /* Check command line arguments. */
	        for(int i=0;i<args.length;i++) {
			char []command; // Command line argument.
			char []value;	// Value of arguement.
			
			command=args[i].toCharArray();
			value=new char[command.length-2];
						
			/* Get value of command. */
			for(int j=2;j<command.length;j++)
				value[j-2]=command[j];
			/* Check command. */
			if(command[0]=='-') {
				/* Server. */
				if(command[1]=='P') {
					System.err.println("Polygons found.");
					polygonFilename=new String(value);
				}
			}	
	
		}
		
		/* Open the polygon file. */
		try {in = new FileInputStream(polygonFilename);}
		catch (Exception e){fileFound=false;}
		
		/* Check if the file existed.*/
		if(in!=null)
			fileFound=true;

		/* Read from the flipper's file. */
		if(fileFound) {
			try { 
				login = new ObjectInputStream(in);
			
	 			flippers[LEFT]=(Polygon)login.readObject();
				flippers[RIGHT]=(Polygon)login.readObject();
			}
			catch (Exception e){System.err.println(e);}
		}
		
		/* Make new flippers. */
		if(!fileFound) {
			flippers[LEFT]=new Polygon();
			flippers[RIGHT]=new Polygon();
		}
	}
	
	/** 
	 * Set the parent of this object. Used to set the focus on the canvas for events. 
 	 * 
	 * @param parent Window who has the flippers image.
  	 */
	public void setParent(AdjustableDrawingCanvas parent) {
		this.parent=parent;
	}
	
	/** 
	 * Mouse is pressed add the location of the mouse to the flipper zone. 
	 *
	 * @param event the mouse was pressed. 
	 */
	public void mousePressed(MouseEvent event) {
		flipperAddPoint(event.getX(),event.getY());
	}
	
	/* Generic Mouse Events not used. */
	public void mouseClicked(MouseEvent event) {}
	/* Generic Mouse Events not used. */
	public void mouseReleased(MouseEvent event) {}
	/* Generic Mouse Events not used. */
	public void mouseExited(MouseEvent event) {}

	/* Generic Keyboard Events not used. */
	public void keyPressed(KeyEvent key) {}
	/* Generic Keyboard Events not used. */
	public void keyReleased(KeyEvent key) {}
	
	/** 
	 * When the mouse is window, make sure the canvas has focus.
	 *
	 * @param event Mouse has entered the window.
	 */
	public void mouseEntered(MouseEvent event) {
		parent.requestFocus();
	}
	
	
	/** 
	 * Keys are pressed, check them. 
	 * 
	 * @param key the key pressed.
	 */
	public void keyTyped(KeyEvent key) {
		/* Set the points for the left flipper. */
		if(key.getKeyChar()=='1')
			which=LEFT; 
		/* Set the points for the right flipper. */
		else if(key.getKeyChar()=='2')
			which=RIGHT;
		/* Delete the flipper's points. */
		else if(key.getKeyChar()=='d')
			flipperDelete();
		/* Save the flipper's file. */
		else if(key.getKeyChar()=='s')
			flippersSave();		
	}
	
	/** Delete the points for the flipper. */
	private void flipperDelete() {
		flippers[which]=new Polygon();
	}
	
	/** 
	 * Add a point to the fliper.
	 * 
	 * @param x x location of the point.
	 * @param y y location of the point.
	 */
	private void flipperAddPoint(int x, int y) {
		flippers[which].addPoint(x,y);
	}
	
	/** 
	 * Get the left flipper. 
	 *
	 * @return the left flipper.
	 */
	public Polygon getLeft() {
		return flippers[0];
	}
	
	/**
	 * Get the right flipper. 
	 * 
	 * @return the right flipper.
	 */
	public Polygon getRight() {
		return flippers[1];
	}
	
	/**
	 * Save the flipper's zones. 
	 */
	public void flippersSave() {
	
		/* Open the polygon file. */
		try {out = new FileOutputStream(polygonFilename);}
		catch (Exception e){fileFound=false;}
		
		/* Check if the file existed.*/
		if(out!=null)
			fileFound=true;

		if(fileFound) {
			try { 
				logout = new ObjectOutputStream(out);
			
	 			logout.writeObject(flippers[LEFT]);
				logout.writeObject(flippers[RIGHT]);
			}
			catch (Exception e){System.err.println(e);}
		}
	}
}
 
