package pinball.player.handmouse;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;
import pinball.player.draw.*;

import java.awt.*;
import java.util.*;


/** Makes the mouse pointer move to the person's hand. */
public class HandMouseStrategy extends Strategy {
	/** Last hand location. */
	private int lastX=-1, lastY=-1;
	
	/** Window location. */
	private int windowWidth, windowHeight;
	
	/** The area in the image to look for the mouse. */
	HandMouseZone zone;
	
	static {
                    System.loadLibrary("HandMouse");
		    
	}
	
	/** Click a mouse button. 
	 * @param b button to press.
	 */
	protected native void click(int b);
	
	/** Move the pointer to a new location. 
	 * @param x x location.
	 * @param y y location.
	 */
	protected native void movePointer(int x,int y);
	
	/** Initalize the mouse pointer. */
	protected native int init();

	/** Get the width of the root window (the desktop size). */
	protected native int width();

	/** Get the height of the root window (the desktop size). */	
	protected native int height();
	
	/** Time from last call. */
	protected long lastTime=0;
	
	/** Is the mouse out of the view. */
	protected boolean mouseOut=true;

	/** Time for a mouse click. */
	protected long clickTime=500;

	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
  	 * @param camera the camera for the system.
	 */		
	public HandMouseStrategy(Buffer buffer, TextArea gameStats, Camera camera) {
		this.stub=new HandMouseStub(buffer,gameStats,this,camera);
		id="handmouse";
		if(init()!=0) {
			System.err.println("Problem setting up the display");
			System.exit(1);
		}
			
		windowWidth=width();
		windowHeight=height();
		System.err.println("Root Window Size: "+windowWidth+ ","+windowHeight);
		lastTime=System.currentTimeMillis();
		Zone.read();
	}

	/** Have a decision to move the mouse pointer. */ 				
	public void flippersCheck() {	
		decision();
	}
	/**
	 * Move the mouse pointer to the location that the bean is pointing to. If the person click's the
	 * bean on/off then it's a click.
	 */
	public void decision() {
		
		/* Laser point is seen. */
		if(stub.blipx()!=-1) {
			
			/** The mouse is not inside the mouse zone. */
			if(!Zone.get().contains(stub.blipx(),stub.blipy())) {
				return;
			}
			
			/** Ratio of the image to the screen size. */
			float xFactor, yFactor;
			
			xFactor=windowWidth;
			xFactor/=Zone.get().width;
			
			yFactor=windowHeight;
			yFactor/=Zone.get().height;
			
			//System.err.println(xFactor+" "+yFactor);

			int xlocation=stub.blipx()-Zone.get().x, ylocation=stub.blipy()-Zone.get().y;
						
			movePointer((int)(xlocation*xFactor),(int)(ylocation*yFactor));

			/* Laser pointer was turned off and then on within the click time, so the user wants to
			 click the mouse. The system moves the pointer back to the previous location and then clicks. 
			
			if(lastX!=-1 && lastY!=-1 && mouseOut && System.currentTimeMillis()-lastTime<clickTime) {
				
				movePointer((int)(lastX*xFactor),(int)(lastY*yFactor));
				click(1);
				System.err.println("CLICK");
				
			}		
			 */		

			//System.err.println("Size Factor: "+xFactor+" "+yFactor);
	
			lastTime=System.currentTimeMillis();
	
			mouseOut=false;
			lastX=xlocation;
			lastY=ylocation;
		}
		
		/* Laser point is not seen. */		
		if(stub.blipx()==-1) {
			mouseOut=true;
		}
		
	}
	
	/**
	 * Draw's nothing on the screen.
	 *
	 * @param g the graphics to draw to.
	 * @param cavasWidth the width of the canvas.
	 * @param canvcasHeight the hight of the canvas.
	 */
	public void draw(Graphics g, int canvasWidth, int canvasHeight) {
	}

}
