package pinball.player.network;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import java.net.*;


/**
 * The network player's strategy.
 * This sends the location of the ball, and other data over to the client.
 * This starts up the server.
 */
public class NetworkPlayerStrategy extends Strategy {
	/** Sends data over the network. */
	protected MyServer server;
		
	/** The size of the buffers in the system. */
	protected int bufferSize;
	
		
	/** 
	 * The strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
	 * @param camera the camera for the system
 	 *
	 */		 	 
	public NetworkPlayerStrategy(Buffer buffer,TextArea gameStats, Camera camera) {
		id="NetworkPlayer";
		this.stub=new Stub(buffer,gameStats,this,camera);
	}
	
	public NetworkPlayerStrategy(){}

	/** Startup the server. */
	public void custom() {
		server=new MyServer();
		new NetworkListener(server,this.stub).start();
		bufferSize=GameOptions.bufferSize();
	}	

	/**
	 * A buffer is available for sending over the network.
	 */
	public void bufferAvailable() {
		/* If no server, release any waiting buffers. */
		if(server==null) {
			buffer=stub.getNextBuffer();
			stub.releaseAllBuffers();
			return;
		}
				
		/* If the server is online and this is the first time a client's connecting, 
		   sent the configuration information. */
		if(server.isOnline() && server.goingOnline()) {
			sendDefaultInfo();
			/* This message tells the ticket server that the client is connected. */
			System.out.println("CONNECTION ESTABLISHED");
			server.goingOnline(false);
		}											

		playStatus();
		buffer=stub.getNextBuffer();
		sendBlip();
		sendBuffInfo();

		/* If the client requested a frame, send it. */
		if(stub.sendFrame()) {
			transmitFrame();
			stub.sendFrame(false);
		}
		
		buildBufferInfo();		
		
		/* If the server is offline, release the lock so the program can still function. */
		if (!server.isOnline()) {
			stub.releaseAllBuffers();
			server.goingOnline(true);
		}			
					
				
	}

	/**
	 * Send the default information to the client. 
	 */
	private void sendDefaultInfo() {
		leftFlipperLocation();
		rightFlipperLocation();
		flipperSpace();
		outZone();
		flipperZone();
		topBottom();
		resetTime();
		foundFlippers();					
	}
 

	/**
	 * Send ball in play's status.
	 */		
	private void playStatus() {
		server.transmit("playStatus "+new Boolean(stub.statsBallInPlay()).toString());
	}
	
	/**
	 * Send flipper's space.
	 */		
	private void flipperSpace() {
		server.transmit("flipperSpace");
	}

	/**
	 * Send top & bottom of play area.
	 */				
	private void topBottom() {
		server.transmit("topBottom");
	}

	/**
	 * Send flipper reset time.
	 */				
	private void resetTime() {
		server.transmit("resetTime "+resetTime);
	}
	
	/**
	 * Send left flipper location.
	 */				
	private void leftFlipperLocation() {
		server.transmit("leftFlipperLocation "+stub.leftX()+" "+stub.leftY());
	}
	
	/**
	 * Send right flipper location.
	 */						
	private void rightFlipperLocation() {
		server.transmit("rightFlipperLocation "+stub.rightX()+" "+stub.rightY());
	}

	/**
	 * Found flippers
	 */						
	private void foundFlippers() {
		server.transmit("foundFlippers");
	}


	/**
	 * Send outzone of image.
	 */						
	private void outZone() {
		server.transmit("outzone "+outZone);
	}		

	/**
	 * Send flipper zone of image.
	 */						
	private void flipperZone() {
		server.transmit("flipperZone "+flipperZone);
	}
	
	/**
	 * Send height of image.
	 */				
	private void height() {
		server.transmit("height "+stub.height());
	}
	
	/**
	 * Send ball's location.
	 */						
	private void sendBlip() {
		server.transmit("blipLocation "+stub.blipx()+" "+stub.blipy()+
			" "+stub.XOffset()+" "+stub.YOffset());
	}

	/**
	 * Send the frame from the camera.
	 */						
	private void transmitFrame() {
		String data=new String(stub.getRawFrame());

		server.transmit("frame "+data);
	}
	
	/**
	 * Send signal to build the buffer on the remote side.
	 */						
	private void buildBufferInfo(){
		server.transmit("buildBufferInfo");
	}	
	
	/**
	 * Send buffer's information.
	 */						
	private void sendBuffInfo() {
		server.transmit("buffInfo "+
			stub.id()+" "+
			stub.number()+" "+				
			stub.width()+" "+
			stub.height()+
			objects());
	}
	
	/**
	 * Send buffer's objects.
	 */				
	private String objects() {
		/* The objects seen. */ 
		Vector blips=stub.getObjects();
	
		/* The blip working on. */
		Blip blip;						
		
		String objects=new String();		
		
		for(int t=0;t<blips.size();t++) {
			blip=(Blip)blips.elementAt(t);
			objects+=new String(" "+blip.xLocation()+" "+blip.yLocation()+" "+blip.total());				
		}
			if(blips.size()==0)
			objects=" ";
		return objects;
	}																					
		
}
