package pinball.player.remote;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.standard.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import java.net.*;

/**
 * This allows a player to play over the network. A player's strategy that's used
 * as a client, changes the Stub to a RemoteStub. When the player sends commands
 * to flip the flippers, the RemoteStub sends the commands over the network. Likewise
 * when the player wants to read data, such as ball location, the RemoteStub gets the
 * data from the networks. RemoteStub has the same commands as Stub, so the player
 * can't tell it's playing on the network. As far as it can tell it's playing locally.
 */
public class RemoteStub extends Stub {


	/** Ball in play status. */
	protected boolean ballInPlay=false;
	
	/** Locations of the flippers. */
	protected Location left,right;

	/** X location of ball. */
	protected int blipx;

	/** Y location of ball. */
	protected int blipy;

	/** X image offset. */
	protected int xoffset;
	
	/** Y image offset. */	
	protected int yoffset;
	
	/** 
	 * Location from bottom of screen to be considered out of play.
	 * Gets value from fromBottom. 
	 */
	protected int outZone;
	
	/** Y location of begin flipper area. */
	protected int flipperZone;
	
	/** Strategy used. */
	protected Strategy strategy;
	
	/** Buffer in use. */
	protected Buffer buffer;
	
	/** Buffer info in use. */
	protected BufferInfo buffinfo;
	
	/** Left flipper up state. */
	protected boolean leftState=false;

	/** Right flipper up state. */	
	protected boolean rightState=false;
	
	/** Time flipper was last up. */
	protected long leftTime, rightTime;
	
	/** Send data to the server. */
	protected MyClient sender;
		
	/** Image size. */
	protected int width, height;
	 
	/** Frame number. */
	protected int number;

	/** Buffer's id. */
	protected int id;
	
	/** Objects found in the frame. */
	protected Vector objects;

	/** Time ping was sent. */
	protected long networkTime;
	
	/** The image from the camera. */
	protected byte image[]=null;
    	
    	/**
	 * The stub for the strategy.
	 *
	 * @param strategy the strategy thread. 
	 */	
	public RemoteStub(Strategy strategy) {
		System.err.println("Remote Stub");
		setupImage();
		this.strategy=strategy;
		leftTime=System.currentTimeMillis();
		rightTime=System.currentTimeMillis();
		sender=new MyClient();
		strategy.setQueue(queue);
		new RemoteListener(this,sender).start();
	}	

    	/**
	 * The stub for the strategy.
	 *
	 * @param strategy the strategy thread.
	 * @param client the client who's connected to the server. 
	 */	
	public RemoteStub(Strategy strategy, MyClient client) {
		System.err.println("Remote Stub");
		setupImage();
		this.strategy=strategy;
		leftTime=System.currentTimeMillis();
		rightTime=System.currentTimeMillis();
		sender=client;
		strategy.setQueue(queue);
		new RemoteListener(this,sender).start();
	}	
	
	
    	/**
	 * The stub for the strategy.
	 *
	 * @param buffer The first buffer for the information buffers.
	 * @param gameStats the area on the screen to write stats info.
	 * @param the strategy thread. 
	 */	
	public RemoteStub(Buffer buffer, TextArea gameStats, Strategy strategy) {
		System.err.println("Remote Stub");
		setupImage();
		this.strategy=strategy;
		leftTime=System.currentTimeMillis();
		rightTime=System.currentTimeMillis();
		sender=new MyClient();
		strategy.setQueue(queue);
		new RemoteListener(this,sender).start();
	}
	
	/**
	 * Do nothing in the main thread. 
	 */
	public void run(){}
	
	/**
	 * Get the ball in play status.
	 *
	 * @param tokens the data from the command.
	 */
	public void playStatus(StringTokenizer tokens) {
		ballInPlay=new Boolean(tokens.nextToken()).booleanValue();		
	}
	
	/**
	 * Get the in play time.
	 *
	 * @param tokens the data from the command.
	 */	
	public void resetTime(StringTokenizer tokens) {
		strategy.resetTime(new Integer(tokens.nextToken()).intValue());
	}
	
	/**
	 * Get the left flipper location.
	 *
	 * @param tokens the data from the command.
	 */
	public void leftFlipperLocation(StringTokenizer tokens) {
		int x=new Integer(tokens.nextToken()).intValue();
		int y=new Integer(tokens.nextToken()).intValue();
		
		left=new Location(x,y);
	}

	/**
	 * Get the right flipper location.
	 *
	 * @param tokens the data from the command.
	 */	
	public void rightFlipperLocation(StringTokenizer tokens) {
		int x=new Integer(tokens.nextToken()).intValue();
		int y=new Integer(tokens.nextToken()).intValue();
		
		right=new Location(x,y);
	}
	
	/**
	 * Get the out zone area.
	 *
	 * @param tokens the data from the command.
	 */	
	public void outZone(StringTokenizer tokens) {
		outZone=new Integer(tokens.nextToken()).intValue();
		bottom=outZone;
	}

	/**
	 * Get the flipper zone.
	 *
	 * @param tokens the data from the command.
	 */	
	public void flipperZone(StringTokenizer tokens) {
		flipperZone=new Integer(tokens.nextToken()).intValue();
	}
	 
	/**
	 * Get the height of the image.
	 *
	 * @param tokens the data from the command.
	 */
	public void height(StringTokenizer tokens) {
		height=new Integer(tokens.nextToken()).intValue();
	}
	
	/**
	 * Get the ball's location.
	 *
	 * @param tokens the data from the command.
	 */	
	public void blipLocation(StringTokenizer tokens) {
		blipx=new Integer(tokens.nextToken()).intValue();
		blipy=new Integer(tokens.nextToken()).intValue();
		xoffset=new Integer(tokens.nextToken()).intValue();
		yoffset=new Integer(tokens.nextToken()).intValue();
	}

	/**
	 * Get the buffer.
	 *
	 * @param tokens the data from the command.
	 */
	public void buffInfo(StringTokenizer tokens) {
		id=new Integer(tokens.nextToken()).intValue();
		number=new Integer(tokens.nextToken()).intValue();
		width=new Integer(tokens.nextToken()).intValue();
		height=new Integer(tokens.nextToken()).intValue();				
		
		int x,y,total;
		Blip blip;
		
		objects = new Vector();

		for(int amount=(tokens.countTokens()/3);amount>0;amount--) {
			x=new Integer(tokens.nextToken()).intValue();
			y=new Integer(tokens.nextToken()).intValue();
			total=new Integer(tokens.nextToken()).intValue();
			
			blip=new Blip(x,y,total);
			
			objects.addElement(blip);
		}
	}
	
	/**
	 * Sets the image from the camera. 
	 */
	public void setImage(String data) {
		/* Copy out the frame data from the data sent over the network.
		   The first 5 characters is the word "frame" the rest of the 
		   data is the actual frame data. 
		*/
		String frame=data.substring(5,data.length());
			
		this.image=frame.getBytes();
	}

	/**
	 * Build the buffer.
	 */
	public void buildBufferInfo() {
//		System.err.println("Called");
		
		buffer=new Buffer(id);
		buffinfo=buffer.info();
		
		if(image!=null) {
			buffinfo.set(new CameraFrame(number,image,width,height,xoffset,yoffset));
			image=null;
		}
		
		buffinfo.blip().x(blipx);
		buffinfo.blip().y(blipy);
		buffinfo.offsets(xoffset,yoffset);
		buffinfo.number(number);
		buffinfo.width(width);
		buffinfo.height(height);
		buffinfo.setBlips(null,objects);
		
		setBuffer(buffer);
				
		addBuffer(buffer);		
			
 		strategy.bufferAvailable();				

	}

	/** 
	 * Transmit ping command. 
	 */		
	public void ping() {
		networkTime=System.currentTimeMillis();
		sender.transmit("PING "+networkTime);
	}

	/** 
	 * Request flipper statistics.
	 */		
	public void flipStatsRequest() {
		sender.transmit("FLIPSTATS");
	}


	/** 
	 * Request the time. 
	 */		
	public void timeRequest() {
		sender.transmit("TIME");
	}
	
	/**
	 * Get the pong command.
	 *
	 * @param tokens the data from the command.
	 */	
	public void pong(StringTokenizer tokens) {
		long pingTime=new Long(tokens.nextToken()).longValue();
		long roundTrip=System.currentTimeMillis()-pingTime;
		
		System.err.println("Ping Transfer Time = "+roundTrip+"ms");

		if(buffinfo!=null)
			logit(buffinfo.number()+" TransferTime="+roundTrip,1);
	}
	
	/**
	 * Send the flipper space.
	 */	
	public void flipperSpace() {
			sender.transmit("flipperSpace "+GameOptions.flipperSpace());
	}

	/**
	 * Send the top and bottom area of game play.
 	 */
	public void topBottom() {
			sender.transmit("topBottom "+top()+" "+bottom());
	}
 	
 	/**
 	 * Set the left flipper state.  	 
 	 *
 	 * @param b the flipper state.
 	 */
	public void leftFlipperState(boolean b) {		
		leftState=b;	
		if(b) {	
			sender.transmit("leftFlip");
			leftTime=System.currentTimeMillis();
		}
		else sender.transmit("leftReset");
	}
	
 	/**
 	 * Set the right flipper state.  	 
 	 *
 	 * @param b the flipper state.
 	 */
	public void rightFlipperState(boolean b) {
		rightState=b;
		if(b) {
			sender.transmit("rightFlip");
			rightTime=System.currentTimeMillis();
		}
		else sender.transmit("rightReset");
	} 
	
	/** 
	 * Check if ball is in play.
	 *
	 * @return ball's in play status.
	 */
	public boolean statsBallInPlay() {
		return ballInPlay;
	}
	
	/** 
	 * X location of left flipper.
	 *
	 * @return left flipper's x location.
	 */
	public int leftX() {
		return left.x();
	}

	/** 
	 * Y location of left flipper.
	 *
	 * @return left flipper's y location.
	 */
	public int leftY() {
		return left.y();
	}
	
	/** 
	 * X location of right flipper.
	 *
	 * @return right flipper's x location.
	 */
	public int rightX() {
		return right.x();
	}

	/** 
	 * Y location of right flipper.
	 *
	 * @return right flipper's y location.
	 */
	public int rightY() {
		return right.y();
	}
	
	/**
	 * Get the flipper's zone.
	 *
	 * @return flipper zone.
	 */
	public int flipperZone() {
		return flipperZone;
	}

	/**
	 * Get the flipper's outzone.
	 *
	 * @return flipper outzone.
	 */	
	public int outZone() {
		return outZone;
	}

	/**
	 * Get the frame number.
	 *
	 * @return frame number.
	 */
	public int number() {
		return buffinfo.number();
	}
	
	/**
	 * Should the left flipper be reset. Has it been up long enough. 
	 *
	 * @param done Has the flipper been up at least this long?
	 *
	 * @return should the flipper be reset or not.
	 */
	public boolean leftFlipperReset(long time) {
		/* Elapsed time flipper was up. */
		long elapsed;
		
		elapsed=System.currentTimeMillis()-leftTime;

		/* Was it up long enough? */
		if(elapsed>=time) {			
			leftTime=System.currentTimeMillis();
			return true;
		}
		else return false;		
	}
	
	/**
	 * Should the right flipper be reset. Has it been up long enough. 
	 *
	 * @param done Has the flipper been up at least this long?
	 *
	 * @return should the flipper be reset or not.
	 */
	public boolean rightFlipperReset(long time) {
		/* Elapsed time flipper was up. */
		long elapsed;
		
		elapsed=System.currentTimeMillis()-rightTime;	

		/* Was it up long enough? */
		if(elapsed>=time) {			
			rightTime=System.currentTimeMillis();
			return true;
		}
		else return false;
	}
	
	/**
	 * Is the left flipper down.
	 *
	 * @return flipper's down status. 
	 */		
	public boolean isLeftUp() {
		return leftState;
	}
	
	/**
	 * Is the right flipper down.
	 *
	 * @return flipper's down status. 
	 */	
	public boolean isRightUp() {
		return rightState;
	}
	
	/**
	 * Is the left flipper down.
	 *
	 * @return flipper's down status. 
	 */
	public boolean isLeftDown() {
		return !leftState;
	}
	
	/**
	 * Is the right flipper down.
	 *
	 * @return flipper's down status. 
	 */
	public boolean isRightDown() {
		return !rightState;
	}
	
	
	/**
	 * Get the height. 
	 *
	 * @return height.
	 */
	public int height() {
		return height;
	}
			
	/**
	 * Release the lock on the information buffer. 
	 */	
	public void releaseLock(Buffer b) {
		sender.transmit("releaseLock "+b.id());
	}
	
	/** 
	 * Set the resize flag and include resize information. 
	 *
	 * @param x x size.
	 * @param y y size.
	 * @param OX x offset.
	 * @param OY y offset.
	 */	
	public void resize(int w,int d, int xo, int yo) {
		sender.transmit("RESIZE "+w+" "+d+" "+xo+" "+yo);
	}
	
	/**
	 * Write to the log.
	 *
	 * @param s String to write.
	 * @param p priority level.
	 */
	public void logit(String s,int p) {		
		sender.transmit("LOG "+s+" "+p);
	}

	/**
	 * Write to the header of the log.
	 *
	 * @param s String to write.
	 */	
	public void headerInfo(String s) {		
		sender.transmit("HEADER "+s);
	}

	/**
	 * Get a frame from the camera. 
	 */
	 public void requestFrame() {
	 	sender.transmit("frame");
	}
	
	/**
	 * Release all buffers.
	 */
	public void releaseAllBuffers() {
		sender.transmit("releaseAllBuffers");
	}
	
	/**
	 * The client is going offline. 
	 */
	public void offline() {
		sender.transmit("QUIT");
		System.exit(0);
	}
 
} 
