package pinball.player.standard;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.network.*;
import pinball.player.remote.*;
import pinball.part.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/**
 * The standard player class for the pinball game.
 * This class defines what threads a player will use in the system. It also determines what mode 
 * the player will be in. When starting up it takes the following command line arguments to define it's mode.<P>
 * -s for server mode.<P>
 * -c for client mode.<P>	 
 * <P>
 * If none of these are specified, then the player is in local mode.<P>
 * <P>
 * There are four other command line arguements allowed.<P>
 * <P>
 * -LPath for the log file. Path is the location to write the log file too. You must include a / at the end
 * of the path location.<P>
 *<P>
 * -lpath will do the log file without saving the frames from the camera.
 *<P>
 * -Ffile is the location of the flipper file. This stores the positions of the flippers. Use this if you
 * plan on restarting the system many times, to skip the flipper location phase. 	
 *  
 */
public class StdPlayer {
	
	/* Defines the type of play for the strategy. */
	
	/** Server mode. */
	protected static final String SERVER="server";
	
	/** Client mode. */
	protected static final String CLIENT="client";
	
	/** Local mode. */
	protected static final String LOCAL="local";
	
	/** How the strategy is to play. Default is local. */
	protected String how=LOCAL;
		
	/** The main part of the game. */
	protected JavaQcam javaQcam;
	
	/** The player of the game. */
	protected StdPlayer player;
	
	/** The camera for the player. */
	protected Camera camera;

	/**
	 * The id String for the player. 
	 * Used in the log file to keep track of which player played.
	 */
	protected static String id="std";
	
	/**
	 * Determines how the strategy is to play.
	 *
	 * @param args Command line arguments.
	 */
	public void howPlay(String args[]) {		
		/* Check command line arguments. */
	        for(int i=0;i<args.length;i++) {
			char []command; // Command line argument.
			char []value;	// Value of arguement.
			
			command=args[i].toCharArray();
			value=new char[command.length-2];
						
			/* Get value of command. */
			for(int j=2;j<command.length;j++)
				value[j-2]=command[j];
			/* Check command. */
			if(command[0]=='-') {
				/* Server. */
				if(command[1]=='S' || command[1]=='s') {
					System.err.println("Server mode.");
					how=SERVER;
				}

				/* Client. */
				if(command[1]=='C' || command[1]=='c') {
					System.err.println("Client mode.");
					how=CLIENT;
				}
				
				/* Tamper proof mode turned on. */
				if(command[1]=='T')
					System.setSecurityManager(new PinballSecurityManager(new String(value)));
				
			}
		}
	}
	

	/** 
	 * Makes the player. 
	 *
	 * @param player Player to be.
	 * @param args Command line arguments.
	 */
	public void makeIt(StdPlayer player, String args[]) {

		/* Find out how the player is to be set up. */
		player.howPlay(args);
	
		/* Client mode. */
		if(player.how().equals(CLIENT)) {
			GameOptions.clientMode();		 	
			Strategy rp = player.clientStrategy(null,new TextArea());
		
			Thread remote=new Thread(rp);
		
			remote.start();			
		}
				
		
		/* Server mode. */
		else if(player.how().equals(SERVER)) {
			camera=new Camera(true);
			new JavaQcam(args,player,"s");
		}
		
		/* Local mode. */
		else if(player.how().equals(LOCAL)) {
			camera=new Camera(true);
			new JavaQcam(args,player,"l");
		}
		
		/* Incorrect command. Wrong strategy thread. */
		else {
			System.err.println("Error: Wrong Strategy Thread Command");
			System.exit(1);
		}
		
		System.err.println("Player: "+player.id());

	}
	
	/** 
	 * How the player is setup.
	 *
	 * @return Player mode status.
	 */
	public String how() {
		return how;
	}

	/** Get the camera from the system. */
	public Camera getCamera() {
		return camera;
	}	
		
	/**
	 * Make the information buffers. Return the first of the list.
	 *
	 * @param amount The amount of buffers to create.
	 * @param Qcam The camera.
	 *
	 * @return the first of the list.
	 */
	public Buffer makeBuffers(int amount, Camera Qcam) {
	        Buffer first=null; /* First buffer. */
		BufferInfo buffinfo=null; /* Buffer information. */
		Buffer buffer=null; /* The buffer being worked on. */
	        
	        /* Create the firstr buffer, with it's info. */
 		first=new Buffer(1);
 		buffer=first;
		buffinfo=buffer.info();
		   
		System.err.println("Making "+amount+" buffers.");
		
		/* If less than 1 buffer. */
		if(amount<1) {
			System.err.println("Amount of frame buffer is less than 1!");
			System.exit(1);
		}
		
		/* Make the rest of the linked list of buffers. */
		for (int i=1;i<amount;i++) {
		
				/* Set up the information. */
				buffer.next(new Buffer(i+1));
				buffinfo.cameraInfo(Qcam.frameSize(),Qcam.xSize(),Qcam.ySize());
				buffer=buffer.next();
				buffinfo=buffer.info();
		}
	
		/* Close the link. */				
		buffer.next(first);
		
		/* Set up it's information. */
		buffinfo.cameraInfo(Qcam.frameSize(),Qcam.xSize(),Qcam.ySize());
		
		
		/* Return the first buffer in the list. */
		return first;
	}


	/** 
	 * Start up the program. Takes command line arguments.
   	 * Takes the following command line arguments:
	 * -s for server mode.
	 * -c for client mode.	 
	 * -S for server mode.
	 *
	 */
	public static void main(String args[]) {		
		StdPlayer player=new StdPlayer();
		player.makeIt(player,args);
	}

	
	/** 
	 * Get the analyze thread for the player. 
	 *
	 * @param buffer the first buffer in the series of buffers for the system.
	 * 
	 * @return Analyze thread.
	 */
	public Analyze analyze (Buffer buffer) {
		return new Analyze(buffer); 
	}

	/** 
	 * Get the frame grabber thread for the player. 
	 *
	 * @param Qcam the camera.
 	 * @param buffer the first buffer in the series of buffers for the system.
 	 * @param window the window to draw to.
 	 *
 	 * @return frame grabber thread.
	 */

	public FrameGrabber frameGrabber(Camera Qcam, Buffer buffer) {
		return new FrameGrabber(Qcam,buffer);
	}

	/** 
	 * Get the strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
	 * @param camera the camera for the system.
 	 *
 	 * @return strategy thread.
	 */
	public Strategy localStrategy(Buffer buffer,TextArea gameStats, Camera camera) {
		return new Strategy(buffer,gameStats,camera);
	}
	
	
	/** 
	 * Get the strategy thread for the player.
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
 	 *
 	 * @return strategy thread.
	 */
	public Strategy clientStrategy(Buffer buffer,TextArea gameStats) {
		return new RemotePlayerStrategy(buffer,gameStats);
	}
	
	/** 
	 * Get the strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
	 * @param camera the camera for the system.
 	 *
 	 * @return strategy thread.
	 */
	public Strategy serverStrategy(Buffer buffer,TextArea gameStats, Camera camera) {
		return new NetworkPlayerStrategy(buffer,gameStats,camera);
	}

	/** 
	 * Get the post thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
 	 *
 	 * @return strategy thread.
	 */
	public Post post(Buffer buffer){
		return new Post(buffer);
	}

	/**
	 * Get id of player.
	 *
	 * @return id of player.
	 */
	public String id() {
		return id;
	}

}
