package pinball.player.standardV2;

import pinball.*;
import pinball.buffer.*; 
import pinball.part.*;
import pinball.server.*;
import pinball.player.*;
import pinball.player.multipleblip.*;
import pinball.player.holdhotspot.*;
import pinball.player.standard.*;
import pinball.player.network.*;
import pinball.player.remote.*;
import pinball.part.*;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;

/**
 * The standard player version 2 for the pinball game.
 * It has a separate thread for the image capturing. If a image is not grabbed,
 * the the buffer just passes through the system doing nothing along the way.
 * 
 * This class defines what threads a player will use in the system. It also determines what mode 
 * the player will be in. When starting up it takes the following command line arguments to define it's mode.<P>
 * -s for server mode.<P>
 * -c for client mode.<P>	 
 * <P>
 * If none of these are specified, then the player is in local mode.<P>
 * <P>
 * There are four other command line arguements allowed.<P>
 * <P>
 * -LPath for the log file. Path is the location to write the log file too. You must include a / at the end
 * of the path location.<P>
 *<P>
 * -lpath will do the log file without saving the frames from the camera.
 *<P>
 * -Ffile is the location of the flipper file. This stores the positions of the flippers. Use this if you
 * plan on restarting the system many times, to skip the flipper location phase. 	
 *  
 */
public class StdPlayer2 extends StdPlayer{
	protected StandardInputs inputDevices;

	/**
	 * The id String for the player. 
	 * Used in the log file to keep track of which player played.
	 */
	protected static String id="std2";
	
	/** 
	 * Start up the program. Takes command line arguments.
   	 * Takes the following command line arguments:
	 * -s for server mode.
	 * -c for client mode.	 
	 */
	public static void main(String args[]) {		
		StdPlayer player=new StdPlayer2();
		player.makeIt(player,args);
	} 
		
		
	/**
	 * Make the information buffers. Return the first of the list.
	 *
	 * @param amount The amount of buffers to create.
	 * @param Qcam The camera.
	 *
	 * @return the first of the list.
	 */
	public Buffer makeBuffers(int amount, Camera Qcam) {
		inputDevices=new StandardInputs(Qcam);
		
	        Buffer first=null; /* First buffer. */
		BufferInfo buffinfo=null; /* Buffer information. */
		Buffer buffer=null; /* The buffer being worked on. */
	        
	        /* Create the firstr buffer, with it's info. */
 		first=new Buffer(1);
 		buffer=first;
		buffinfo=buffer.info();
		   
		System.err.println("Making "+amount+" buffers.");
		
		/* If less than 1 buffer. */
		if(amount<1) {
			System.err.println("Amount of frame buffer is less than 1!");
			System.exit(1);
		}
		
		/* Make the rest of the linked list of buffers. */
		for (int i=1;i<amount;i++) {
		
				/* Set up the information. */
				buffer.next(new Buffer(i+1));
				buffinfo.cameraInfo(Qcam.frameSize(),Qcam.xSize(),Qcam.ySize());
				buffer=buffer.next();
				buffinfo=buffer.info();
		}
	
		/* Close the link. */				
		buffer.next(first);
		
		/* Set up it's information. */
		buffinfo.cameraInfo(Qcam.frameSize(),Qcam.xSize(),Qcam.ySize());
		
		
		/* Return the first buffer in the list. */
		return first;
	}
	
	/** 
	 * Get the analyze thread for the player. 
	 *
	 * @param buffer the first buffer in the series of buffers for the system.
	 * 
	 * @return Analyze thread.
	 */
	public Analyze analyze (Buffer buffer) {
		return new Analyze(buffer); 
	}

	/** 
	 * Get the frame grabber thread for the player. 
	 *
	 * @param Qcam the camera.
 	 * @param buffer the first buffer in the series of buffers for the system.
 	 * @param window the window to draw to.
 	 *
 	 * @return frame grabber thread.
	 */

	public FrameGrabber frameGrabber(Camera Qcam, Buffer buffer) {
		return new Input(buffer,inputDevices);
	}

	/** 
	 * Get the strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
	 * @param camera the camera for the system.
 	 *
 	 * @return strategy thread.
	 */
	public Strategy localStrategy(Buffer buffer,TextArea gameStats, Camera camera) {
		return new StandardV2Strategy(buffer,gameStats,camera);
	}
	
	
	/** 
	 * Get the strategy thread for the player.
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
 	 *
 	 * @return strategy thread.
	 */
	public Strategy clientStrategy(Buffer buffer,TextArea gameStats) {
		return new RemotePlayerStrategy(buffer,gameStats);
	}
	
	/** 
	 * Get the strategy thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
	 * @param gameStats the area to write the game stats to.
	 * @param camera the camera for the system.
 	 *
 	 * @return strategy thread.
	 */
	public Strategy serverStrategy(Buffer buffer,TextArea gameStats, Camera camera) {
		return new NetworkPlayerStrategy(buffer,gameStats,camera);
	}

	/** 
	 * Get the post thread for the player. 
	 *
 	 * @param buffer the first buffer in the series of buffers for the system.
 	 *
 	 * @return strategy thread.
	 */
	public Post post(Buffer buffer){
		return new Post(buffer);
	}

	/**
	 * Get id of player.
	 *
	 * @return id of player.
	 */
	public String id() {
		return id;
	}

}
